import pygame
from OpenGL import GL,GLU

class SpriteImage(object):
	__allocatedImages = {}
	
	def __init__(self, orig_image, use_mipmapping = False):
		def Make2n(number):
			i = 1
			while i < number:
				i*=2
			return i
		
		self.mipMapped = use_mipmapping
		
		#create an image with OpenGL-compatible size
		if not hasattr(orig_image, 'blit'):
			orig_image = pygame.image.load(orig_image)
		self.size = orig_image.get_size()
		self.tex_size = tuple(map(Make2n,self.size))
		tex_image = pygame.Surface(self.tex_size,pygame.SRCALPHA,32)
		tex_image.blit(orig_image,(0,0))
		del orig_image
		
		#create the opengl texture object
		self.tex_name = GL.glGenTextures(1)
		GL.glBindTexture(GL.GL_TEXTURE_2D,self.tex_name)
		
		#copy the image to the opengl texture
		string = pygame.image.tostring(tex_image,"RGBA")
		if use_mipmapping:
			GLU.gluBuild2DMipmaps(GL.GL_TEXTURE_2D, GL.GL_RGBA,
				self.tex_size[0],self.tex_size[1],
				GL.GL_RGBA,GL.GL_UNSIGNED_BYTE,string)
		else:
			GL.glTexImage2D(GL.GL_TEXTURE_2D, 0, GL.GL_RGBA,
				self.tex_size[0],self.tex_size[1],0,
				GL.GL_RGBA,GL.GL_UNSIGNED_BYTE,string)
		
		#clean
		GL.glBindTexture(GL.GL_TEXTURE_2D,0)
		
	def getSize(self):
		return self.size
	
	def getSpriteImage(imageFile):
		if imageFile not in SpriteImage.__allocatedImages:
			SpriteImage.__allocatedImages[imageFile] = SpriteImage(imageFile)
		return SpriteImage.__allocatedImages[imageFile]
	getSpriteImage = staticmethod(getSpriteImage)
	
	def getTextSpriteImage(text, textColor = (255,255,255), font_size = 16):
		selected_font = pygame.font.SysFont('Arial', font_size, True)
		renderedText = selected_font.render(text, True, textColor)
		return SpriteImage(renderedText)
	getTextSpriteImage = staticmethod(getTextSpriteImage)

	def draw(self):
		"draws the image to (0,0) - (width,height)"
		
		#bind the texture
		GL.glEnable(GL.GL_TEXTURE_2D)
		GL.glBindTexture(GL.GL_TEXTURE_2D,self.tex_name)
		
		if self.mipMapped:
			GL.glTexParameteri(GL.GL_TEXTURE_2D, GL.GL_TEXTURE_MIN_FILTER, GL.GL_LINEAR_MIPMAP_LINEAR)
		else:
			GL.glTexParameteri(GL.GL_TEXTURE_2D, GL.GL_TEXTURE_MIN_FILTER, GL.GL_LINEAR)
		GL.glTexParameteri(GL.GL_TEXTURE_2D, GL.GL_TEXTURE_MAG_FILTER, GL.GL_LINEAR)
		
		#prepare the matrices
		GL.glPushMatrix() #modelview
		GL.glScalef(self.size[0],self.size[1],1)
		
		GL.glMatrixMode(GL.GL_TEXTURE)
		GL.glPushMatrix()
		GL.glLoadIdentity()
		GL.glScalef(float(self.size[0])/self.tex_size[0], float(self.size[1])/self.tex_size[1], 1)
		
		#draw
		GL.glBegin(GL.GL_QUADS)
		for coord in [(0,0),(0,1),(1,1),(1,0)]:
			GL.glTexCoord2f(*coord)
			GL.glVertex2f(*coord)
		GL.glEnd()
		
		#restore the matrices
		GL.glMatrixMode(GL.GL_TEXTURE)
		GL.glPopMatrix()
		GL.glMatrixMode(GL.GL_MODELVIEW)
		GL.glPopMatrix()
		
		#unbind the texture
		GL.glDisable(GL.GL_TEXTURE_2D)
		GL.glBindTexture(GL.GL_TEXTURE_2D,0)
	
	def __del__(self):
		if GL.glDeleteTextures:
			GL.glDeleteTextures([self.tex_name])

def set_color(color):
	GL.glColor4f(*color)
	if color != [1,1,1,1]:
		GL.glTexEnvi(GL.GL_TEXTURE_ENV, GL.GL_TEXTURE_ENV_MODE, GL.GL_MODULATE)
	else:
		GL.glTexEnvi(GL.GL_TEXTURE_ENV, GL.GL_TEXTURE_ENV_MODE, GL.GL_REPLACE)

